@@ -3,10 +3,11 @@ import asyncio
 import os
 import shutil
 from types import TracebackType
-from typing import Any, AsyncGenerator, Callable, Coroutine, Dict, Generator, IO, Iterable, Optional, Tuple, Type, Union
+from typing import Any, AsyncGenerator, Callable, Coroutine, Dict, Generator, IO, Iterable, Optional, Tuple, Type, Union, List
 from aiofiles.threadpool.binary import AsyncBufferedIOBase
 
-import aiohttp, aiofiles
+import aiohttp
+import aiofiles
 from aiohttp.client import DEFAULT_TIMEOUT
 
 from aiodav.utils import WebDavXmlUtils
@@ -15,11 +16,12 @@ from aiodav.exceptions import *
 
 log = logging.getLogger(__name__)
 
+
 class Client(object):
     """
     WebDAV Client
 
-    Parameters:    
+    Parameters:
         hostname (``str``):
             Url for WebDAV server should contain protocol (HTTP or HTTPS) and ip address or domain name.
             Example: `https://webdav.server.com`
@@ -52,8 +54,8 @@ class Client(object):
             Set a password to use in proxy authentication.
 
         chunk_size (``int``, *optional*):
-            Size of buffer used to transfer data from/to server. This data will be 
-            loaded in memory. This parameter afect the progress callback in 
+            Size of buffer used to transfer data from/to server. This data will be
+            loaded in memory. This parameter afect the progress callback in
             download/upload functions. Default is 65536 bytes (65K)
     """
 
@@ -115,18 +117,19 @@ class Client(object):
         self._chunk_size = chunk_size if chunk_size else 65536
         self._proxy = proxy
         self._proxy_auth = aiohttp.BasicAuth(proxy_user, proxy_password) if (
-                proxy_user and proxy_password) else None
+            proxy_user and proxy_password) else None
         self._insecure = insecure
         self.session = aiohttp.ClientSession(
-            loop = loop,
-            timeout = aiohttp.ClientTimeout(total=timeout) if timeout else DEFAULT_TIMEOUT,
-            auth = aiohttp.BasicAuth(login, password) if (
+            loop=loop,
+            timeout=aiohttp.ClientTimeout(total=timeout) if timeout else DEFAULT_TIMEOUT,
+            auth=aiohttp.BasicAuth(login, password) if (
                 login and password) else None,
+            connector=kwargs.get('connector', None)
         )
 
     def _get_headers(
-        self, 
-        action: str, 
+        self,
+        action: str,
         headers_ext: Optional[Dict[str, str]] = None
     ) -> Dict[str, str]:
         headers = None
@@ -145,36 +148,36 @@ class Client(object):
         return headers
 
     def _get_url(
-        self, 
+        self,
         path: Union[str, "os.PathLike[str]"]
     ) -> str:
         return f"{self._hostname}{self._root}{path}"
 
     def get_full_path(
-        self, 
+        self,
         urn: Urn
     ) -> str:
         return f"{self._root}{urn.path()}"
 
     async def _execute_request(self,
-        action: str,
-        path: Union[str, "os.PathLike[str]"],
-        data: Optional[Any] = None,
-        headers_ext: Optional[Dict[str, str]] = None
-    ) -> aiohttp.ClientResponse:
+                               action: str,
+                               path: Union[str, "os.PathLike[str]"],
+                               data: Optional[Any] = None,
+                               headers_ext: Optional[Dict[str, str]] = None
+                               ) -> aiohttp.ClientResponse:
         try:
             if self.session.auth:
                 await self.session.get(url=self._hostname)  # (Re)Authenticates against the proxy
 
             response = await self.session.request(
-                method = Client.DEFAULT_REQUESTS[action],
-                url = self._get_url(path),
-                headers = self._get_headers(action, headers_ext),
-                data = data,
-                proxy = self._proxy,
-                proxy_auth = self._proxy_auth,
+                method=Client.DEFAULT_REQUESTS[action],
+                url=self._get_url(path),
+                headers=self._get_headers(action, headers_ext),
+                data=data,
+                proxy=self._proxy,
+                proxy_auth=self._proxy_auth,
                 # chunked = self._chunk_size,
-                ssl = False if self._insecure else None
+                ssl=False if self._insecure else None
             )
 
             if response.status == 507:
@@ -187,9 +190,9 @@ class Client(object):
                 raise ResponseErrorCode(url=self._get_url(path), code=response.status, message=response.content)
 
             return response
-        except aiohttp.ConnectionError:
+        except aiohttp.ClientConnectionError:
             raise NoConnection(self._hostname)
-        except aiohttp.RequestException as re:
+        except aiohttp.ClientResponseError as re:
             raise ConnectionException(re)
         except Exception as e:
             raise e
@@ -223,12 +226,12 @@ class Client(object):
         self,
         path: Optional[Union[str, "os.PathLike[str]"]] = ROOT,
         get_info: Optional[bool] = False
-    ) -> Union[list[str], list[Dict[str, str]]]:
+    ) -> Union[List[str], List[Dict[str, str]]]:
         """
         Returns list of nested files and directories for remote WebDAV directory by path.
         More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_PROPFIN
 
-        Parameters:    
+        Parameters:
             path (``str``):
                 Path to remote directory.
 
@@ -236,8 +239,8 @@ class Client(object):
                 Set true to get more information like cmd 'ls -l'.
 
         Returns:
-            List of :obj:`str` | List of :obj:`Dict[str, str]`: On success, if get_info=False it returns 
-                list of nested file or directory names, otherwise it returns list of information, the 
+            List of :obj:`str` | List of :obj:`Dict[str, str]`: On success, if get_info=False it returns
+                list of nested file or directory names, otherwise it returns list of information, the
                 information is a dictionary and it values with following keys:
 
                 `created`: date of resource creation,
@@ -266,26 +269,26 @@ class Client(object):
     async def free(self) -> int:
         """
         Returns an amount of free space on remote WebDAV server.
-        More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_PROPFIND       
+        More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_PROPFIND
 
         Returns:
             :obj:`int`: An amount of free space in bytes.
         """
-        
+
         data = WebDavXmlUtils.create_free_space_request_content()
         response = await self._execute_request(action='free', path='', data=data)
         text = await response.text()
         return WebDavXmlUtils.parse_free_space_response(text, self._hostname)
 
     async def exists(
-        self, 
+        self,
         path: Union[str, "os.PathLike[str]"]
     ) -> bool:
         """
         Checks an existence of remote resource on WebDAV server by remote path.
         More information you can find by link http://webdav.org/specs/rfc4918.html#rfc.section.9.4
 
-        Parameters:    
+        Parameters:
             path (``str``):
                 Path to remote resource.
 
@@ -304,21 +307,21 @@ class Client(object):
         return (int(response.status) == 200)
 
     async def create_directory(
-        self, 
+        self,
         path: Union[str, "os.PathLike[str]"]
     ) -> bool:
         """
         Makes new directory on WebDAV server.
         More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_MKCOL
 
-        Parameters:    
+        Parameters:
             path (``str``):
                 Path to remote directory.
 
          Returns:
             :obj:`bool`: True if request executed with code 200 or 201 and False otherwise.
         """
-        
+
         directory_urn = Urn(path, directory=True)
         if not (await self.exists(directory_urn.parent())):
             raise RemoteParentNotFound(directory_urn.path())
@@ -331,29 +334,29 @@ class Client(object):
         return response.status in (200, 201)
 
     async def _check_remote_resource(
-        self, 
-        path: Union[str, "os.PathLike[str]"], 
+        self,
+        path: Union[str, "os.PathLike[str]"],
         urn: Urn
     ) -> None:
         if not (await self.exists(urn.path())) and not (await self.exists(Urn(path, directory=True).path())):
             raise RemoteResourceNotFound(path)
 
     async def is_directory(
-        self, 
+        self,
         path: Union[str, "os.PathLike[str]"]
     ) -> bool:
         """
         Checks is the remote resource directory.
         More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_PROPFINDL
 
-        Parameters:    
+        Parameters:
             path (``str``):
                 Path to remote directory.
 
          Returns:
             :obj:`bool`: True in case the remote resource is directory and False otherwise.
         """
-        
+
         urn = Urn(path)
         parent_urn = Urn(urn.parent())
         await self._check_remote_resource(path, urn)
@@ -364,14 +367,14 @@ class Client(object):
         return WebDavXmlUtils.parse_is_dir_response(content=text, path=path, hostname=self._hostname)
 
     async def info(
-        self, 
+        self,
         path: Union[str, "os.PathLike[str]"]
     ) -> Dict[str, str]:
         """
         Gets information about resource on WebDAV.
         More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_PROPFIND
 
-        Parameters:    
+        Parameters:
             path (``str``):
                 Path to remote resource.
 
@@ -384,7 +387,7 @@ class Client(object):
                 `modified`: date of resource modification,
                 `etag`: etag of resource.
         """
-        
+
         urn = Urn(path)
         await self._check_remote_resource(path, urn)
 
@@ -394,7 +397,7 @@ class Client(object):
         return WebDavXmlUtils.parse_info_response(content=text, path=path, hostname=self._hostname)
 
     async def unlink(
-        self, 
+        self,
         path: Union[str, "os.PathLike[str]"]
     ) -> None:
         """
@@ -402,7 +405,7 @@ class Client(object):
         with original library.
         More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_DELETE
 
-        Parameters:    
+        Parameters:
             path (``str``):
                 Path to remote resource.
         """
@@ -411,7 +414,7 @@ class Client(object):
         await self._execute_request(action='clean', path=urn.quote())
 
     async def delete(
-        self, 
+        self,
         path: Union[str, "os.PathLike[str]"]
     ) -> None:
         """
@@ -419,7 +422,7 @@ class Client(object):
         with original library.
         More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_DELETE
 
-        Parameters:    
+        Parameters:
             path (``str``):
                 Path to remote resource.
         """
@@ -436,10 +439,10 @@ class Client(object):
         Moves resource from one place to another on WebDAV server.
         More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_MOVE
 
-        Parameters:    
+        Parameters:
             source (``str``):
                 The path to resource which will be moved.
-            
+
             destination (``str``):
                 the path where resource will be moved.
 
@@ -472,17 +475,17 @@ class Client(object):
         Copies resource from one place to another on WebDAV server.
         More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_COPY
 
-        Parameters:    
+        Parameters:
             source (``str``):
                 The path to resource which will be copied.
-            
+
             destination (``str``):
                 the path where resource will be copied.
 
             depth (``int``, *optional*):
                 Folder depth to copy. Default is 1
         """
-        
+
         urn_from = Urn(source)
         if not (await self.exists(urn_from.path())):
             raise RemoteResourceNotFound(urn_from.path())
@@ -500,14 +503,14 @@ class Client(object):
         await self._execute_request(action='copy', path=urn_from.quote(), headers_ext=headers)
 
     async def download_iter(
-        self, 
+        self,
         path: Union[str, "os.PathLike[str]"]
     ) -> Generator[bytes, None, None]:
         """
         Downloads file from server and return content in generator.
         More information you can find by link http://webdav.org/specs/rfc4918.html#rfc.section.9.4
 
-        Parameters:    
+        Parameters:
             path (``str``):
                 The path to remote resource
 
@@ -544,7 +547,7 @@ class Client(object):
         Downloads file from server and writes it in buffer.
         More information you can find by link http://webdav.org/specs/rfc4918.html#rfc.section.9.4
 
-        Parameters:    
+        Parameters:
             path (``str``):
                 The path to remote resource
 
@@ -607,8 +610,9 @@ class Client(object):
                 await buffer.write(block)
             else:
                 buffer.write(block)
-                
+
             current += len(block)
+
             if callable(progress):
                 if asyncio.iscoroutinefunction(progress):
                     await progress(current, total, *progress_args)
@@ -626,7 +630,7 @@ class Client(object):
         Downloads file from server and write to a file.
         More information you can find by link http://webdav.org/specs/rfc4918.html#rfc.section.9.4
 
-        Parameters:    
+        Parameters:
             remote_path (``str``):
                 The path to remote file for downloading.
 
@@ -665,7 +669,7 @@ class Client(object):
                 await client.download_to('/path/to/file.zip', '/home/file.zip', progress=progress)
                 ...
         """
-        
+
         async with aiofiles.open(local_path, 'wb') as file:
             await self.download_to(remote_path, file, progress=progress, progress_args=progress_args)
 
@@ -683,7 +687,7 @@ class Client(object):
 
         WARNING: Destructive method
 
-        Parameters:    
+        Parameters:
             remote_path (``str``):
                 The path to directory for downloading form WebDAV server.
 
@@ -711,7 +715,7 @@ class Client(object):
                 Extra custom arguments as defined in the ``progress_args`` parameter.
                 You can either keep ``*args`` or add every single extra argument in your function signature.
         """
-        
+
         urn = Urn(remote_path, directory=True)
 
         if not (await self.is_directory(urn.path())):
@@ -741,7 +745,7 @@ class Client(object):
 
         WARNING: DESTRUCTIVE METHOD (This method can call `self.download_directory`)
 
-        Parameters:    
+        Parameters:
             remote_path (``str``):
                 The path to remote resource for downloading.
 
@@ -769,7 +773,7 @@ class Client(object):
                 Extra custom arguments as defined in the ``progress_args`` parameter.
                 You can either keep ``*args`` or add every single extra argument in your function signature.
         """
-        
+
         urn = Urn(remote_path)
         if (await self.is_directory(urn.path())):
             await self.download_directory(local_path=local_path, remote_path=remote_path, progress=progress, progress_args=progress_args)
@@ -788,7 +792,7 @@ class Client(object):
         Uploads file from buffer to remote path on WebDAV server.
         More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_PUT
 
-        Parameters:    
+        Parameters:
             path (``str``):
                 The path to remote resource
 
@@ -829,18 +833,18 @@ class Client(object):
                     await client.upload_to('/path/to/file.zip', file, progress=progress)
                 ...
         """
-        
+
         urn = Urn(path)
         if urn.is_dir():
             raise OptionNotValid(name="path", value=path)
 
         if not (await self.exists(urn.parent())):
-            raise RemoteParentNotFound(urn.path())      
-        
+            raise RemoteParentNotFound(urn.path())
+
         if callable(progress) and not asyncio.iscoroutinefunction(buffer):
             async def file_sender(buff: IO):
                 current = 0
-                
+
                 if asyncio.iscoroutinefunction(progress):
                     await progress(current, buffer_size, *progress_args)
                 else:
@@ -851,14 +855,14 @@ class Client(object):
                         else buffer.read(self._chunk_size)
                     if not chunk:
                         break
-                    
+
                     current += len(chunk)
-                    
+
                     if asyncio.iscoroutinefunction(progress):
                         await progress(current, buffer_size, *progress_args)
                     else:
                         progress(current, buffer_size, *progress_args)
-                    yield chunk                
+                    yield chunk
 
             await self._execute_request(action='upload', path=urn.quote(), data=file_sender(buffer))
         else:
@@ -875,7 +879,7 @@ class Client(object):
         Uploads file to remote path on WebDAV server. File should be 2Gb or less.
         More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_PUT
 
-        Parameters:    
+        Parameters:
             remote_path (``str``):
                 The path to uploading file on WebDAV server.
 
@@ -914,11 +918,11 @@ class Client(object):
                 await client.upload_file('/path/to/file.zip', 'file.zip', progress=progress)
                 ...
         """
-        
+
         async with aiofiles.open(local_path, 'rb') as file:
             size = os.path.getsize(local_path)
-            await self.upload_to(path=remote_path, buffer=file, buffer_size=size, 
-                    progress=progress, progress_args=progress_args)
+            await self.upload_to(path=remote_path, buffer=file, buffer_size=size,
+                                 progress=progress, progress_args=progress_args)
 
     async def upload_directory(
         self,
@@ -928,14 +932,14 @@ class Client(object):
         progress_args: Optional[Tuple] = ()
     ) -> None:
         """
-        Uploads directory to remote path on WebDAV server. In case directory is exist 
-        on remote server it will delete it and then upload directory with nested files 
+        Uploads directory to remote path on WebDAV server. In case directory is exist
+        on remote server it will delete it and then upload directory with nested files
         and directories.
         More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_PUT
 
         WARNING: DESTRUCTIVE METHOD
 
-        Parameters:    
+        Parameters:
             remote_path (``str``):
                 The path to directory for uploading on WebDAV server.
 
@@ -963,7 +967,7 @@ class Client(object):
                 Extra custom arguments as defined in the ``progress_args`` parameter.
                 You can either keep ``*args`` or add every single extra argument in your function signature.
         """
-        
+
         urn = Urn(remote_path, directory=True)
         if not urn.is_dir():
             raise OptionNotValid(name="remote_path", value=remote_path)
@@ -998,7 +1002,7 @@ class Client(object):
 
         WARNING: DESTRUCTIVE METHOD
 
-        Parameters:    
+        Parameters:
             remote_path (``str``):
                 The path to resource for uploading on WebDAV server.
 
@@ -1041,7 +1045,7 @@ class Client(object):
         Gets metadata property of remote resource on WebDAV server.
         More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_PROPFIND
 
-        Parameters:    
+        Parameters:
             path (``str``):
                 Path to remote directory.
 
@@ -1052,7 +1056,7 @@ class Client(object):
         Returns:
             :obj:`str` | None: The value of property or None if property is not found.
         """
-        
+
         urn = Urn(path)
         if not (await self.exists(urn.path())):
             raise RemoteResourceNotFound(urn.path())
@@ -1071,7 +1075,7 @@ class Client(object):
         Sets metadata property of remote resource on WebDAV server.
         More information you can find by link http://webdav.org/specs/rfc4918.html#METHOD_PROPPATCH
 
-        Parameters:    
+        Parameters:
             path (``str``):
                 Path to remote directory.
 
@@ -1080,7 +1084,7 @@ class Client(object):
                     `name`: the name of property which will be set,
                     `value`: (optional) the value of property which will be set. Defaults is empty string.
         """
-        
+
         urn = Urn(path)
         if not (await self.check(urn.path())):
             raise RemoteResourceNotFound(urn.path())
@@ -1097,13 +1101,13 @@ class Client(object):
         await self.session.close()
 
     def resource(
-        self, 
+        self,
         path: Union[str, "os.PathLike[str]"]
     ) -> "Resource":
         """
         Get associated resource from path
 
-        Parameters:    
+        Parameters:
             path (``str``):
                 Path to remote directory.
 
@@ -1112,7 +1116,8 @@ class Client(object):
         """
 
         urn = Urn(path)
-        return Resource(self, urn)  
+        return Resource(self, urn)
+
 
 class Resource(object):
     """
@@ -1120,8 +1125,8 @@ class Resource(object):
     """
 
     def __init__(
-        self, 
-        client: Client, 
+        self,
+        client: Client,
         urn: Urn
     ) -> None:
         self.client = client
@@ -1140,13 +1145,13 @@ class Resource(object):
         return await self.client.is_directory(self.urn.path())
 
     async def rename(
-        self, 
+        self,
         name: str
     ) -> None:
         """
         Rename the resource.
 
-        Parameters:    
+        Parameters:
             name (``str``):
                 New name to resource.
         """
@@ -1159,13 +1164,13 @@ class Resource(object):
         self.urn = Urn(new_path)
 
     async def move(
-        self, 
+        self,
         path: Union[str, "os.PathLike[str]"]
     ) -> None:
         """
         Move the resource to new path.
 
-        Parameters:    
+        Parameters:
             path (``str``):
                 New path of the resource.
         """
@@ -1180,7 +1185,7 @@ class Resource(object):
         """
         Copy the resource to a another path.
 
-        Parameters:    
+        Parameters:
             path (``str``):
                 The path where resource will be copied.
 
@@ -1192,13 +1197,13 @@ class Resource(object):
         return Resource(self.client, urn)
 
     async def info(
-        self, 
+        self,
         filter: Optional[Iterable[str]] = None
     ) -> Dict[str, str]:
         """
         Get a dictionary with resource information.
 
-        Parameters:    
+        Parameters:
             filter (``Iterable[str]``, *optional*):
                 If filter is not `None` then only return properties
                 contained in filter iterable.
@@ -1234,4 +1239,4 @@ class Resource(object):
             :obj:`bool`: True if the resource exists else return False.
         """
 
-        return await self.client.exists(self.urn.path())
\ No newline at end of file
+        return await self.client.exists(self.urn.path())
